@@ -53,6 +53,7 @@ extern PyObject *_CBOR2_str_FrozenDict;
 extern PyObject *_CBOR2_str_fromordinal;
 extern PyObject *_CBOR2_str_getvalue;
 extern PyObject *_CBOR2_str_groups;
+extern PyObject *_CBOR2_str_imag;
 extern PyObject *_CBOR2_str_ip_address;
 extern PyObject *_CBOR2_str_ip_network;
 extern PyObject *_CBOR2_str_is_infinite;
@@ -69,6 +70,7 @@ extern PyObject *_CBOR2_str_parsestr;
 extern PyObject *_CBOR2_str_pattern;
 extern PyObject *_CBOR2_str_prefixlen;
 extern PyObject *_CBOR2_str_read;
+extern PyObject *_CBOR2_str_real;
 extern PyObject *_CBOR2_str_s;
 extern PyObject *_CBOR2_str_timestamp;
 extern PyObject *_CBOR2_str_toordinal;
